//*****************************************************************************
// File Name	: clock.c
// 
// Title		: Lab #3
// Revision		: 1.0
// Notes		:	
// Target MCU	: Atmel AVRmega323 
//*****************************************************************************

 
//----- Include Files ---------------------------------------------------------
#include <io.h>		// include I/O definitions (port names, pin names, etc)
#include <sig-avr.h>	// include "signal" names (interrupt names)
#include <interrupt.h>	// include interrupt support
#include <progmem.h>

#include "global.h"	// include our global settings
#include "rprintf.h"	// include printf function library
#include "timer.h"	// include timer function library
#include "uart.h"	// include timer function library

#include "./lcd.h"        // LCD driver
#include "./pcf8574.h"    // pcf8574 driver
#include "./ata.h"        // ata driver
#include "./fat.h"
#include "./mas3587f.h"

//----- Defines ---------------------------------------------------------------

// Button definition
#define MODE_BUTTON 0
#define PLAY_BUTTON 1
#define FWD_BUTTON 2
#define REV_BUTTON 3
#define STOP_BUTTON 4
#define ADJ_BUTTON 5

// Definition of button thresholds
#define PROGRAM_BUTTON_THRESHOLD 20 
#define ALARM_BUTTON_THRESHOLD 20 
#define UP_BUTTON_THRESHOLD 8 
#define DOWN_BUTTON_THRESHOLD 8 

// Different modes of the clock
#define DISPLAY_TIME 0
#define PLAY_MP3 1
#define RECORD_MP3 2
#define DELETE_MP3 3
#define SET_PROGRAM 4
#define SET_TIME 5
#define SYNC_MODE 6
#define NUM_MODES 2

// Amount of time to blink.
#define BLINK_COUNT 16

// Custom characters used in LCD
#define PROGRAM_ICON 0
#define ALARM_ICON   1
#define WAVE_R1   2
#define WAVE_R2   3
#define WAVE_R3   4
#define WAVE_L1   5
#define WAVE_L2   6
#define WAVE_L3   7


//----- Functions -------------------------------------------------------------
void updateTime(void);
void printTime(unsigned short hours, unsigned short minutes, unsigned short seconds);
void program_clock(void);
void set_alarm(void);
void program_alarm(void);
void play_alarm(void);
void play_note_a(unsigned short index);
void play_note_b(unsigned short index);
void customChars(void);
void displayAlarmPlay(void);
void set_contrast(void);

//----- Global Variables ------------------------------------------------------
// current time
static unsigned short hours;
static unsigned short minutes;
static unsigned short seconds;
static unsigned short date;
static unsigned short month;
static unsigned short year;

// other variables used throughout
static unsigned short lastseconds;
static unsigned short count;
static unsigned short program_count;
static unsigned short alarm_count;
static unsigned short up_count;
static unsigned short down_count;
static unsigned short temp_count;
static unsigned short time_modulo;
static unsigned short mode;
static unsigned short alarm_icon_state;
static unsigned short contrast;

static unsigned short buttons;
static unsigned short lastButtons;
static unsigned short buttonSet;
static unsigned short buttonRelease;

static unsigned short numFiles;
static unsigned short currentFileNum;
static unsigned short lastFileNum;

static unsigned short displayOffset;
static unsigned short overflowLCD;

// Character map for the LCD screen
u08 __attribute__  ((progmem)) lcd_char_map[] = { 	
  0xFF, 0xF1, 0xF5, 0xF1, 0xF7, 0xF7, 0xF7, 0xFF,  
  0xE0, 0xE4, 0xEE, 0xEE, 0xEE, 0xFF, 0xE4, 0xE0,
  0xE0, 0xE0, 0xF0, 0xE8, 0xE8, 0xF0, 0xE0, 0xE0,
  0xE0, 0xE4, 0xE2, 0xE2, 0xE2, 0xE2, 0xE4, 0xE0,
  0xE4, 0xE2, 0xE1, 0xE1, 0xE1, 0xE1, 0xE2, 0xE4,
  0xE0, 0xE0, 0xE1, 0xE2, 0xE2, 0xE1, 0xE0, 0xE0,
  0xE0, 0xE4, 0xE8, 0xE8, 0xE8, 0xE8, 0xE4, 0xE0,
  0xE4, 0xE8, 0xF0, 0xF0, 0xF0, 0xF0, 0xE8, 0xE4 
};

static unsigned short silly1;
static unsigned short silly2;
static u16 silly3;
static u32 silly4;

//----- Begin Code ------------------------------------------------------------
int main(void)
{
  unsigned long current_cluster=2;
  u16 i;

  preCharge();
  
  // initialize the AVRlib libraries
  timerInit(); 			// initialize the timer system
  uartInit();                             // initialize the UART (serial port)
  uartSetBaudRate(57600);          // set the baud rate to 9600
  
  sbi(ASSR, AS2); // Use 32khz crystal
  timerAttach(TIMER2OVERFLOW_INT, updateTime);
  timer2SetPrescaler(0x02); // 16 interrupts per second

  timerPause(100);

  i2cInit();


  timerPause(100);
  timerPause(100);
  timerPause(100);
  timerPause(100);
  timerPause(100);
  timerPause(100);
  timerPause(100);
  timerPause(100);
  timerPause(100);
  rprintfInit(uartSendByte);   


  MASreadreg(CONTROL_REG, &silly4);
  rprintf("CONTROL: ");
  rprintfNum(16, 4, FALSE, '0', silly4);
  rprintf("\r\n");
  MASreadreg(DCCF_REG, &silly4);
  rprintf("DCCF: ");
  rprintfNum(16, 4, FALSE, '0', silly4);
  rprintf("\r\n");
  MASreadreg(DCFR_REG, &silly4);
  rprintf("DCFR: ");
  rprintfNum(16, 4, FALSE, '0', silly4);
  rprintf("\r\n");

  initMAS();

  rprintf("CONTROL: ");
  rprintfNum(16, 4, FALSE, '0', silly4);
  rprintf("\r\n");
 
  rprintf("07F0: ");
  silly3 = 0x07F0;
  MASreadD0(silly3, &silly4);
  rprintfu32(silly4);
  rprintf("\r\n");
  
  rprintf("07F3: ");
  silly3 = 0x07F3;
  MASreadD0(silly3, &silly4);
  rprintfu32(silly4);
  rprintf("\r\n");

  rprintf("VOLUME: ");
  silly3 = 0x0010;
  MASreadCODEC(silly3, &silly4);
  rprintfu16(silly4);
  rprintf("\r\n");

  silly4 = 0x7300;
  MASwriteCODEC(silly3, &silly4);

  silly3 = 0x07F6;
  silly4 = 0x0000000C;
  MASwriteD0(silly3, &silly4);

  rprintf("RUNNING: ");
  MASreadD0(silly3, &silly4);
  rprintfu32(silly4);
  rprintf("\r\n");
  


  /*
  lcdInit();                    // initialize the LCD
  lcdSetContrast(100);
  
  
  rprintfInit(lcdDataWrite);
  rprintf("INIT");
  lcdReturn();
  timerPause(100);
    
  rprintfInit(uartSendByte);   
  ataInit();
  ataIdentify(buffer);

  fatInit();

  hours = 12;
  minutes = 0;
  seconds = 0;
  month = 12;
  date = 9;
  year = 22;

  count = 0;
  program_count = 0;
  contrast = 100;
  mode = DISPLAY_TIME;
  displayOffset = 0;

  buttons = 0xFF;
  lastButtons = 0xFF;
  buttonSet = 0x00;
  buttonRelease = 0x00;

  numFiles =  fatGetDirEntry(0);
  currentFileNum = 1;
  lastFileNum = 0;
  rprintf("NUMFILES %d\r\n", numFiles);
  
  sei(); // enable RTC!
  */

  
  // enter endless while loop
  while (1) {
    switch (mode)  {

    case DISPLAY_TIME:
      rprintfInit(lcdDataWrite);
      if (lastseconds != seconds || buttons != lastButtons) {  
	lastseconds = seconds;
	lastButtons = buttons;
	
	lastFileNum = 0;
	lcdSetAddress(0x40);
	printTime(hours, minutes, seconds);
	lcdSetAddress(0x00);
	printDate(date, month, year);
	rprintf(" %x %x %x", buttons, buttonSet, buttonRelease);
	if (seconds == 5 && minutes == 0) {
	  rprintfInit(uartSendByte);   
	  for (i=1; i<= numFiles; i++) {
	    rprintf("GetDirEntry: %d\r\n", fatGetDirEntry(i));
	    getTAG();
	    printFileInfo();
	  }
	}
	
      }
      break;
    case PLAY_MP3:	
      rprintfInit(lcdDataWrite);
      if (numFiles == 0) {
	lcdSetAddress(0x00);
	rprintf("NO FILES TO PLAY        ");
	lcdSetAddress(0x40);
	rprintf("                        ");
      }
      else if (currentFileNum != lastFileNum) {
	lastFileNum = currentFileNum;
	rprintfInit(uartSendByte);   
	rprintf("GetDirEntry: %d\r\n", fatGetDirEntry(currentFileNum));
	getTAG();
	printFileInfo();
	rprintfInit(lcdDataWrite);
	lcdCommandWrite(0x02);
	displayOffset = 0;
	lcdStablePrint("Choose file:            ", displayOffset);
	overflowLCD = printFileInfoLCD();
      }
      if (lastseconds != seconds || buttons != lastButtons) {  
	lastseconds = seconds;
	lastButtons = buttons;
	if (overflowLCD) {
	  lcdCommandWrite(0x18);
	  displayOffset = (displayOffset + 1)%40;
	  lcdStablePrint("Choose file:            ", displayOffset);
	    
	    
	    }
      }
      
      if (buttonSet & 1<<FWD_BUTTON) {
	buttonSet = buttonSet ^ (1<<FWD_BUTTON);
	currentFileNum++;
	if (currentFileNum == numFiles + 1) currentFileNum = 1;
      }
      if (buttonSet & 1<<REV_BUTTON) {
	buttonSet = buttonSet ^ (1<<REV_BUTTON);
	currentFileNum--;
	if (currentFileNum == 0) currentFileNum = numFiles;
      }
      
      break;
    case RECORD_MP3:
      break;
    case DELETE_MP3:
      break;
    case SET_PROGRAM:
      break;
    case SET_TIME:
      break;
    case SYNC_MODE:
      break;
    }
    if (buttonSet & 1<<MODE_BUTTON) {
      buttonSet = buttonSet ^ (1<<MODE_BUTTON);
      mode = (mode+1)%NUM_MODES;
      lcdReset();
      displayOffset = 0;
    }
  }
}



// The interrupt service routine, happens once every 1/16 of a second
// This ISR not only handles the update of the time, it also manages
// the different push button events.  Counters are used to figure out
// how long a button has been pressed.
void updateTime(void)
{
  if (++count == 16) {
    count = 0;	
    if (++seconds == 60) {
      seconds = 0;
      if (++minutes == 60) {
	minutes = 0;
	if (++hours == 24) {
	  hours = 0;
	  if (++date == 32) {
	    month++;
	    date = 1;
	  }
	  else if (date == 31) {
	    if (month == 4 || month == 6 || month == 9 || month == 11) {
	      month++;
	      date = 1;
	    }
	  }
	  else if (date == 30) {
	    if (month == 2) {
	      month++;
	      date = 1;
	    }
	  }
	  else if (date == 29) {
	    if ((month == 2) && (not_leap())) {
	      month++;
	      date = 1;
	    }
	  }
	  if (month == 13) {
	    month = 1;
	    year++;
	  }
	}
      }
    }
  }
  if (seconds % 6 == 0 && count == 0) {
    //MASread(0x6a, &silly1, &silly2);
  }
  if (seconds % 6 == 2 && count == 0) {
    //MASread(0x76, &silly1, &silly2);
    //silly4 = MASreadD0(0x07F3);
  }
  if (seconds % 6 == 4 && count == 0) {
    //MASwrite(0x6A, 0x06, 0x00);
    //silly4 = MASwriteD0(0x07F3, 0x00003E80);
  }

}


void preCharge() {
  u08 i;

  outb(SFIOR, 0x04); // disable pullups
  
  outb(PORTA, 0xFF);
  outb(PORTB, 0x1F);
  outb(PORTC, 0x20);
  outb(PORTD, 0x00);

  outb(DDRA, 0xFF);
  outb(DDRB, 0x1F);
  outb(DDRC, 0x30);
  outb(DDRD, 0x0F);

  for (i=0; i<255; i++);
}


