//*****************************************************************************
// File Name	: clock.c
// 
// Title		: Lab #3
// Revision		: 1.0
// Notes		:	
// Target MCU	: Atmel AVRmega323 
//*****************************************************************************

 
//----- Include Files ---------------------------------------------------------
#include <io.h>		// include I/O definitions (port names, pin names, etc)
#include <sig-avr.h>	// include "signal" names (interrupt names)
#include <interrupt.h>	// include interrupt support

#include "global.h"	// include our global settings
#include "rprintf.h"	// include printf function library
#include "timer.h"	// include timer function library

//----- Defines ---------------------------------------------------------------
// LCD control lines
#define LCD_E 		2
#define LCD_RW 		3
#define LCD_RS 		4
#define LCD_BIAS 	5

// LCD constants
#define LCD_BUSY 	7
#define LCD_SET_C_ADDRESS 	6
#define LCD_SET_D_ADDRESS 	7

// Button definition
#define PROGRAM_BUTTON 4
#define UP_BUTTON 5
#define DOWN_BUTTON 6
#define ALARM_BUTTON 7

// Definition of button thresholds
#define PROGRAM_BUTTON_THRESHOLD 20 
#define ALARM_BUTTON_THRESHOLD 20 
#define UP_BUTTON_THRESHOLD 8 
#define DOWN_BUTTON_THRESHOLD 8 

// Different modes of the clock
#define DISPLAY_TIME 0
#define PROGRAM_CLOCK 1
#define PROGRAM_ALARM 2
#define PLAY_ALARM 3
#define CONTRAST_CONTROL 4

// Amount of time to blink.
#define BLINK_COUNT 16

// Custom characters used in LCD
#define PROGRAM_ICON 0
#define ALARM_ICON   1
#define WAVE_R1   2
#define WAVE_R2   3
#define WAVE_R3   4
#define WAVE_L1   5
#define WAVE_L2   6
#define WAVE_L3   7


//----- Functions -------------------------------------------------------------
void updateTime(void);
void printTime(unsigned short hours, unsigned short minutes, unsigned short seconds);
void lcdInit();
void lcdCommandWrite(u08 data);
void lcdDataWrite(u08 data);
void lcdCheckBusy(void);
void program_clock(void);
void set_alarm(void);
void program_alarm(void);
void play_alarm(void);
void play_note_a(unsigned short index);
void play_note_b(unsigned short index);
void customChars(void);
void displayAlarmPlay(void);
void set_contrast(void);

//----- Global Variables ------------------------------------------------------
// current time
static unsigned short hours;
static unsigned short minutes;
static unsigned short seconds;

// alarm set time
static unsigned short alarm_hours;
static unsigned short alarm_minutes;
static unsigned short alarm_seconds;

// alarm on or off
static unsigned short alarm_status;

// other variables used throughout
static unsigned short lastseconds;
static unsigned short count;
static unsigned short program_count;
static unsigned short alarm_count;
static unsigned short up_count;
static unsigned short down_count;
static unsigned short temp_count;
static unsigned short time_modulo;
static unsigned short mode;
static unsigned short alarm_icon_state;
static unsigned short contrast;

//----- Begin Code ------------------------------------------------------------
int main(void)
{
  
  // initialize the AVRlib libraries
  timerInit(); 			// initialize the timer system
  lcdInit();                    // initialize the LCD
  rprintfInit(lcdDataWrite);	// make all rprintf statements use uart for output
  
  // pause for 1000ms
  timerPause(1000);
  
  // set port B pins 0, 4-7 to all input
  outb(DDRB, 0x0E);
  
  sbi(ASSR, AS2); // Use 32khz crystal
  
  sei();
  
  // AVRlib timer function library handles all timer interrupts
  // use timerAttach function to make doCount run on each overflow of Timer2
  
  timerAttach(TIMER2OVERFLOW_INT, updateTime);
  
  timer2SetPrescaler(0x02); // 16 interrupts per second
  
  customChars(); // initialize custom characters into the LCD
  
  // set up PWM for contrast control
  timer1SetPrescaler(TIMER_CLK_DIV8);
  timer1PWMInit(9);
  cbi(TCCR1A, COM1B1);	// Pascal's code initializes PWM B, so I need
  cbi(TCCR1A, COM1B0);	// to manually disable it.  Thanks Pascal! ;-)
  
  // Good to know the LCD works
  rprintf("Clock initializing...");
  
  
  // reset variable
  hours = 12;
  minutes = 0;
  seconds = 0;
  alarm_hours = 6;
  alarm_minutes = 0;
  alarm_seconds = 0;
  count = 0;
  program_count = 0;
  alarm_count = 0;
  alarm_status = 0;
  contrast = 0;
  mode = DISPLAY_TIME;
  // enter endless while loop
  while (1) {
    if (mode == PROGRAM_CLOCK)
      {
	program_clock();
	mode = DISPLAY_TIME;
      }
    else if (mode == PROGRAM_ALARM)
      {
	set_alarm();
	if (alarm_status) program_alarm();
	mode = DISPLAY_TIME;
      }
    else if (mode == PLAY_ALARM)
      {
	play_alarm();
	mode = DISPLAY_TIME;
      }
    else if (mode == CONTRAST_CONTROL)
      {
	set_contrast();
      }
    else 
      {
	
	// Update LCD only if the seconds have changed
	if (lastseconds != seconds) {			
	  lcdCommandWrite(0x01);
	  printTime(hours, minutes, seconds);
	  if (alarm_status) {
	    rprintf("   ");
	    lcdDataWrite(ALARM_ICON);
	  }
	}
	lastseconds = seconds;
	
	lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));

	// Show button presses for clock set and alarm set
	if (program_count > 0)
	  { 			
	    lcdDataWrite(PROGRAM_ICON);
	    rprintf("CLK");
	  }
	else rprintf("    ");
	if (alarm_count > 0)
	  { 
	    lcdDataWrite(PROGRAM_ICON);
	    rprintf("ALM ");
	    printTime(alarm_hours, alarm_minutes, alarm_seconds);
	  }
	else rprintf("                ");
	
      }

    if (program_count == PROGRAM_BUTTON_THRESHOLD) mode = PROGRAM_CLOCK;
    if (alarm_count == ALARM_BUTTON_THRESHOLD) mode = PROGRAM_ALARM;
   
    // Play the alarm if alarm time equals current time.
    if (hours == alarm_hours 
	&& minutes == alarm_minutes 
	&& seconds == alarm_seconds
	&& alarm_status) 
      mode = PLAY_ALARM;
    
    timerPause(100);
  }
}


// The interrupt service routine, happens once every 1/16 of a second
// This ISR not only handles the update of the time, it also manages
// the different push button events.  Counters are used to figure out
// how long a button has been pressed.
void updateTime(void)
{
  // CONTRAST BUTTON
  if (~inb(PINB)&0x01) mode = CONTRAST_CONTROL;
  
  // Clear alarm if any of 4 main buttons are pressed
  if ((~inb(PINB)&0xF0) && mode == PLAY_ALARM) mode = DISPLAY_TIME;
  
  // PROGRAM BUTTON
  if ((~inb(PINB))&0x80 && program_count != PROGRAM_BUTTON_THRESHOLD) 
    program_count++; 
  if (inb(PINB)&0x80) program_count = 0;
  
  // ALARM BUTTON
  if ((~inb(PINB))&0x10 && alarm_count != ALARM_BUTTON_THRESHOLD) 
    alarm_count++;
  if (inb(PINB)&0x10) alarm_count = 0;
  
  // UP BUTTON
  if (~inb(PINB)&0x40) 
    { 
      up_count++;
      if (up_count % 4 == 1) 
	{temp_count = (temp_count+1)%time_modulo;
	}
    }
  if (inb(PINB)&0x40) up_count = 0;
  
  // DOWN BUTTON
  if (~inb(PINB)&0x20) 
    {
      down_count++;
      if (down_count % 4 == 1) 
	{if (temp_count !=0) temp_count = (temp_count-1)%time_modulo;
	else temp_count = time_modulo-1;
	}
    }
  if (inb(PINB)&0x20) down_count = 0;
  
  // Update time
  if (++count == 16) {
    count = 0;	
    if (++seconds == 60) {
      seconds = 0;
      if (++minutes == 60) {
	minutes = 0;
	if (++hours == 24) {
	  hours = 0;
	}
      }
    }
  }
}

// Nice function to print the time.  Rather straightforward.
void printTime(unsigned short hours, unsigned short minutes, unsigned short seconds)
{
  unsigned short printhours;
  unsigned short am;
  am = 1;
  printhours = hours;
  if (hours == 0) {
    printhours = 12;
  }
  else if (hours == 12) {
    am = 0;
  }	
  else if (hours > 12) {
    printhours = hours - 12;
    am = 0;
  }
  if (printhours < 10) rprintf(" ");
  rprintf("%d:", printhours);
  if (minutes < 10) rprintf("0");
  rprintf("%d:", minutes);
  if (seconds < 10) rprintf("0");
  rprintf("%d", seconds);
  if (am) rprintf("am");
  else rprintf("pm");
}		

// Function to initialize the LCD.
void lcdInit(void) {
	outb(PORTD, 0x00);
	outb(DDRD, 0xFF); // set port d as control
	lcdCommandWrite(0x38); 
	lcdCommandWrite(0x06); 
	lcdCommandWrite(0x0C); 
	lcdCommandWrite(0x01); 
}

// Function to write a command byte to the LCD
void lcdCommandWrite(u08 data) {
	lcdCheckBusy();
	cbi(PORTD, LCD_RS); 	
	cbi(PORTD, LCD_RW);
	outb(DDRA, 0xFF);
	outb(PORTA, data);
	sbi(PORTD, LCD_E);
	delay(200);	
	cbi(PORTD, LCD_E);
	delay(200);	
}

// Function to write a data byte to the LCD
void lcdDataWrite(u08 data) {
	lcdCheckBusy();
	sbi(PORTD, LCD_RS); 	
	cbi(PORTD, LCD_RW);
	outb(DDRA, 0xFF);
	outb(PORTA, data);
	sbi(PORTD, LCD_E);
	delay(200);	
	cbi(PORTD, LCD_E);
	delay(200);	
}

// Function to loop while LCD is busy
void lcdCheckBusy(void) {
  u08 busy;
  do 
    {
      cbi(PORTD, LCD_RS);
      sbi(PORTD, LCD_RW);
      outb(DDRA, 0x00);
      sbi(PORTD, LCD_E);
      delay(200);	
      busy = inb(PINA) & (1<<LCD_BUSY);
      cbi(PORTD, LCD_E);
      delay(200);	
    }	
  while (busy);
}

// Long HOKEY code to program the clock.  I really hate the way this was coded,
// and I wasn't thinking clearly when I was doing this...
void program_clock(void) {
  // blink used to blink the current field
  unsigned short blink;
  // These are the fields that will be programmed
  unsigned short sethours;
  unsigned short setminutes;
  unsigned short setseconds;
  // last_count is used to know when the LCD needs to be updated
  unsigned short last_count;
  sethours = hours;
  setminutes = minutes;
  setseconds = seconds;
  
  blink = 0;
  
  // set the hours
  temp_count = sethours;
  last_count = sethours-1;
  time_modulo = 24;
  lcdCommandWrite(0x01);
  rprintf("Set hour:");		  
  
  // Loop until the user releases the PROGRAM button
  while (program_count != 0) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);		
      sethours = temp_count;
      last_count = temp_count;
    }
  }
  
  while (program_count < 1)  {
    if (temp_count != last_count)  { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      sethours = temp_count;
      last_count = temp_count;
      blink = 0;
    }
    if (blink > BLINK_COUNT/2)  {			
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      rprintf("  ");
      blink++;
    }
    else {
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      blink++;
    }
    if (blink == BLINK_COUNT) blink = 0;
    timerPause(50);
  }
  
  // set the minutes
  temp_count = setminutes;
  last_count = setminutes-1;
  time_modulo = 60;
  lcdCommandWrite(0x01);
  rprintf("Set minutes:");		  
  
  while (program_count != 0) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);		
      setminutes = temp_count;
      last_count = temp_count;
    }
  }
  
  while (program_count < 1) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      setminutes = temp_count;
      last_count = temp_count;
      blink = 0;
    }
    if (blink > BLINK_COUNT/2)  {			
      lcdCommandWrite(0x43|(1<<LCD_SET_D_ADDRESS));
      rprintf("  ");
      blink++;
    }
    else {
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      blink++;
    }
    if (blink == BLINK_COUNT) blink = 0;
    timerPause(50);
  }
  
  // set the seconds
  temp_count = setseconds;
  last_count = setseconds-1;
  time_modulo = 60;
  lcdCommandWrite(0x01);
  rprintf("Set seconds:");		  
  
  while (program_count != 0) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);		
      setseconds = temp_count;
      last_count = temp_count;
    }
  }
  
  while (program_count < 1) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      setseconds = temp_count;
      last_count = temp_count;
      blink = 0;
		}
    if (blink > BLINK_COUNT/2) {			
      lcdCommandWrite(0x46|(1<<LCD_SET_D_ADDRESS));
      rprintf("  ");
      blink++;
    }
    else {
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      blink++;
    }
    if (blink == BLINK_COUNT) blink = 0;
		timerPause(50);
  }
  
  // This is where the values are actually set
  hours = sethours;
  minutes = setminutes;
  seconds = setseconds;
  
  lastseconds = seconds-1;
  
  // Again, loop until the user releases PROGRAM
  while (program_count != 0) {
    if (lastseconds != seconds) {			
      outb(PORTB, 0x00);
      lcdCommandWrite(0x01);
      printTime(hours, minutes, seconds);
      outb(PORTB, 0xFF);
    }
    lastseconds = seconds;
  }
}

// Function to set the alarm on or off.
void set_alarm(void) {
  unsigned short tempstatus;
  tempstatus = !alarm_status;
  while (alarm_count != 0) {
    if (tempstatus != alarm_status) {
      lcdCommandWrite(0x01);
      rprintf("Alarm status:");		  
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      if (alarm_status) {
	rprintf("ON ");
      }
      else rprintf("OFF");
      tempstatus = alarm_status;
    }
  }
  while (alarm_count < 1)  {
    if (tempstatus != alarm_status) {
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      if (alarm_status) {
	rprintf("ON ");
      }
      else rprintf("OFF");
      tempstatus = alarm_status;
    }
    if (up_count) alarm_status = 1;
    if (down_count) alarm_status = 0;	
  }
}

// This function is a carbon copy of program_clock, with minor changes
// I was in a cut and paste mood.
void program_alarm(void) {
  unsigned short blink;
  unsigned short sethours;
  unsigned short setminutes;
  unsigned short setseconds;
  unsigned short last_count;
  sethours = alarm_hours;
  setminutes = alarm_minutes;
  setseconds = alarm_seconds;
  
  blink = 0;
  
  temp_count = sethours;
  last_count = sethours-1;
  time_modulo = 24;
  lcdCommandWrite(0x01);
  rprintf("Set alarm hour:");		  
  
  while (alarm_count != 0) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);		
      sethours = temp_count;
      last_count = temp_count;
    }
  }
  
  while (alarm_count < 1)  {
    if (temp_count != last_count)  { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      sethours = temp_count;
      last_count = temp_count;
      blink = 0;
    }
    if (blink > BLINK_COUNT/2)  {			
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      rprintf("  ");
      blink++;
    }
    else {
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      blink++;
    }
    if (blink == BLINK_COUNT) blink = 0;
    timerPause(50);
  }
  
  temp_count = setminutes;
  last_count = setminutes-1;
  time_modulo = 60;
  lcdCommandWrite(0x01);
  rprintf("Set alarm minutes:");		  
  
  while (alarm_count != 0) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);		
      setminutes = temp_count;
      last_count = temp_count;
    }
  }
  
  while (alarm_count < 1) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      setminutes = temp_count;
      last_count = temp_count;
      blink = 0;
    }
    if (blink > BLINK_COUNT/2)  {			
      lcdCommandWrite(0x43|(1<<LCD_SET_D_ADDRESS));
      rprintf("  ");
      blink++;
    }
    else {
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      blink++;
    }
    if (blink == BLINK_COUNT) blink = 0;
    timerPause(50);
  }
  
  
  temp_count = setseconds;
  last_count = setseconds-1;
  time_modulo = 60;
  lcdCommandWrite(0x01);
  rprintf("Set alarm seconds:");		  
  
  while (alarm_count != 0) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);		
      setseconds = temp_count;
      last_count = temp_count;
    }
  }
  
  while (alarm_count < 1) {
    if (temp_count != last_count) { 
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      setseconds = temp_count;
      last_count = temp_count;
      blink = 0;
    }
    if (blink > BLINK_COUNT/2) {			
      lcdCommandWrite(0x46|(1<<LCD_SET_D_ADDRESS));
      rprintf("  ");
      blink++;
    }
    else {
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      printTime(sethours, setminutes, setseconds);
      blink++;
    }
    if (blink == BLINK_COUNT) blink = 0;
    timerPause(50);
  }
  
  alarm_hours = sethours;
  alarm_minutes = setminutes;
  alarm_seconds = setseconds;
  
  lastseconds = seconds-1;
  
  while (alarm_count != 0) {
    if (lastseconds != seconds) {			
      outb(PORTB, 0x00);
      lcdCommandWrite(0x01);
      printTime(hours, minutes, seconds);
      outb(PORTB, 0xFF);
    }
    lastseconds = seconds;
  }
  
}

// The song we play with the alarm is stored in program memory.  The 
// first field is the TCCR value, the second one is the OCR, and the 
// last one is the duration of the note.
u08 __attribute__  ((progmem)) note_a[] = { 0x1b, 0x31, 0x10,  // 0
					    0x1b, 0x2c, 0x10,  // 3
					    0x1b, 0x27, 0x1f,  // 6
					    0x18, 0x00, 0x01,  // 9
					    0x1b, 0x27, 0x10,  // 12
					    0x1b, 0x2c, 0x10,  // 15
					    0x1b, 0x31, 0x1f,  // 18
					    0x18, 0x00, 0x01,  // 21
					    0x1b, 0x2c, 0xa0,  // 24
					    0x1b, 0x31, 0xf0,  // 27
					    0x18, 0x00, 0x01,  // 30
					    
					    0x1b, 0x31, 0x10,  // 33
					    0x1b, 0x2c, 0x10,  // 36
					    0x1b, 0x27, 0x1f,  // 39
					    0x18, 0x00, 0x01,  // 42
					    0x1b, 0x27, 0xff,  // 45
					    0x1b, 0x27, 0xff,  // 48
					    0x18, 0x00, 0x01,  // 51
					    
					    0x1b, 0x31, 0xa0,  // 54
					    0x1b, 0x2c, 0x10,  // 57
					    0x18, 0x00, 0x00,  // 60
					    0x1b, 0x2c, 0xff,  // 63
					    0x1b, 0x2c, 0xf0,  // 66
					    0x18, 0x00, 0xa0,  // 69
					    
					    0x00 
};

// I had to define a second array because I couldn't fit everything 
// in one 256 byte array, and I was afraid of testing the 16 bit 
// pointer arithmetic
u08 __attribute__  ((progmem)) note_b[] = { 0x1b, 0x31, 0x10,  // 0
					    0x1b, 0x25, 0x04,  // 3
					    0x18, 0x00, 0x00,  // 6
					    0x1b, 0x25, 0x04,  // 9
					    0x18, 0x00, 0x00,  // 12
					    0x1b, 0x25, 0x04,  // 15
					    0x18, 0x00, 0x00,  // 18
					    0x1b, 0x21, 0x04,  // 21
					    0x18, 0x00, 0x10,  // 24
					    
					    0x1b, 0x21, 0x10,  // 27
					    0x1a, 0xef, 0x04,  // 30
					    0x18, 0x00, 0x00,  // 33
					    0x1a, 0xef, 0x04,  // 36
					    0x18, 0x00, 0x00,  // 39
					    0x1b, 0x21, 0x10,  // 42
					    0x1a, 0xef, 0x04,  // 45
					    0x1a, 0xef, 0x04,  // 48
					    0x18, 0x00, 0x00,  // 51
					    0x1a, 0xef, 0x04,  // 54
					    0x18, 0x00, 0x00,  // 57
					    0x1b, 0x21, 0x04,  // 60
					    0x18, 0x00, 0x00,  // 63
					    0x1a, 0xef, 0x2f,  // 66
					    0x18, 0x00, 0x00,  // 69
					    0x1b, 0x21, 0x2f,  // 72
					    0x18, 0x00, 0x00,  // 75
					    0x1b, 0x31, 0xff,  // 78
					    0x1b, 0x31, 0xff,  // 81
					    0x18, 0x00, 0xff,  // 84
					    0x00 			   // 87
};

// This function plays the song, "Linus and Lucy"
void play_alarm(void) {
  int i;
  i = 0;
  unsigned short index;
  unsigned short temp;
  temp = inb(TCCR0);
  
  for (i=0; i<2; i++) 
    {
      for (index = 0; index < 54; index = index + 3) { play_note_a(index); }
      for (index = 0; index < 33; index = index + 3) { play_note_a(index); }
      for (index = 54; index < 72; index = index + 3) { play_note_a(index); }
      for (index = 54; index < 72; index = index + 3) { play_note_a(index); }
      for (index = 0; index < 54; index = index + 3) { play_note_a(index); }
      for (index = 0; index < 33; index = index + 3) { play_note_a(index); }
      for (index = 54; index < 72; index = index + 3) { play_note_a(index); }
      for (index = 54; index < 72; index = index + 3) { play_note_a(index); }
      for (index = 0; index < 78; index = index + 3) { play_note_b(index); }
      for (index = 3; index < 78; index = index + 3) { play_note_b(index); }
      for (index = 3; index < 87; index = index + 3) { play_note_b(index); }
    }
  
  outb(TCCR0, temp);
}

// Function to play notes from note_a array
void play_note_a(unsigned short index) {
  unsigned short i1;
  unsigned short i2;
  
  if (mode == PLAY_ALARM) {
    outb(OCR0, PRG_RDB(&note_a[index+1]));
    outb(TCCR0, PRG_RDB(&note_a[index]));
    // Delay is a little different for silences
    if (PRG_RDB(&note_a[index+2]) == 0) 
      {
	for (i1 = 0; i1!=25; i1++)  {
	  for (i2 = 0; i2!=25; i2++) delay(PRG_RDB(&note_a[index+2]));
	}
      }
    else {
      for (i1 = 0; i1!=50; i1++)  {
	for (i2 = 0; i2!=50; i2++) delay(PRG_RDB(&note_a[index+2]));
      }
    }
    
    // Update the LCD when we get the chance
    if (lastseconds != seconds) {			
      lcdCommandWrite(0x01);
      printTime(hours, minutes, seconds);
      displayAlarmPlay();
    }
    lastseconds = seconds;
  }
}

// essentially the same as the previous function
void play_note_b(unsigned short index) {
  unsigned short i1;
  unsigned short i2;
  if (mode == PLAY_ALARM) {
    outb(OCR0, PRG_RDB(&note_b[index+1]));
    outb(TCCR0, PRG_RDB(&note_b[index]));
    if (PRG_RDB(&note_b[index+2]) == 0) 
      {
	for (i1 = 0; i1!=25; i1++)  {
	  for (i2 = 0; i2!=25; i2++) delay(PRG_RDB(&note_b[index+2]));
	}
      }
    else {
      for (i1 = 0; i1!=50; i1++)  {
	for (i2 = 0; i2!=50; i2++) delay(PRG_RDB(&note_b[index+2]));
      }
    }
    
    if (lastseconds != seconds) {			
      lcdCommandWrite(0x01);
      printTime(hours, minutes, seconds);
      displayAlarmPlay();
    }
    lastseconds = seconds;    
  }
}

// Character map for the LCD screen
u08 __attribute__  ((progmem)) char_map[] = { 	0xFF, 0xF1, 0xF5, 0xF1, 0xF7, 0xF7, 0xF7, 0xFF,  
						0xE0, 0xE4, 0xEE, 0xEE, 0xEE, 0xFF, 0xE4, 0xE0,
						0xE0, 0xE0, 0xF0, 0xE8, 0xE8, 0xF0, 0xE0, 0xE0,
						0xE0, 0xE4, 0xE2, 0xE2, 0xE2, 0xE2, 0xE4, 0xE0,
						0xE4, 0xE2, 0xE1, 0xE1, 0xE1, 0xE1, 0xE2, 0xE4,
						0xE0, 0xE0, 0xE1, 0xE2, 0xE2, 0xE1, 0xE0, 0xE0,
						0xE0, 0xE4, 0xE8, 0xE8, 0xE8, 0xE8, 0xE4, 0xE0,
						0xE4, 0xE8, 0xF0, 0xF0, 0xF0, 0xF0, 0xE8, 0xE4 
};

// Function to load the custom characters into CGRAM
void customChars(void) {
  int i;
  lcdCommandWrite(0x00|(1<<LCD_SET_C_ADDRESS));
  for (i = 0; i<64; i++) 
    {
      lcdDataWrite(PRG_RDB(&char_map[i]));
    }
  lcdCommandWrite(0x01);	
}

// Function to do our alarm animation
void displayAlarmPlay(void) {
  
  rprintf("  ");
  
  if (alarm_icon_state == 0) {
    lcdDataWrite(WAVE_L1);
    lcdDataWrite(ALARM_ICON);
    lcdDataWrite(WAVE_R1);
  }	
  else if (alarm_icon_state == 1) {
    lcdDataWrite(WAVE_L2);
    lcdDataWrite(ALARM_ICON);
    lcdDataWrite(WAVE_R2);
  }	
  else if (alarm_icon_state == 2) {
    lcdDataWrite(WAVE_L3);
    lcdDataWrite(ALARM_ICON);
    lcdDataWrite(WAVE_R3);
  }	
  alarm_icon_state = (alarm_icon_state+1)%3; 
}

// Function to set the contrast
void set_contrast(void) {
  unsigned short sethours;
  unsigned short setminutes;
  unsigned short setseconds;
  
  sethours = hours;
  setminutes = minutes;
  setseconds = seconds;
  
  unsigned short last_count;
  
  temp_count = contrast;
  last_count = contrast-1;
  time_modulo = 61;
  
  while (mode == CONTRAST_CONTROL) {
    if (temp_count != last_count)  { 
      lcdCommandWrite(0x01);
      printTime(sethours, setminutes, setseconds);
      lcdCommandWrite(0x40|(1<<LCD_SET_D_ADDRESS));
      contrast = temp_count;
      last_count = temp_count;
      rprintf("Set Contrast: %d", contrast);		  
      timer1PWMASet(contrast*4);
    }
    timerPause(50);
    if (!(~inb(PINB)&0x01)) mode = DISPLAY_TIME;
  }
}
