/*! \file fat.c \brief FAT16/32 file system driver. */
//*****************************************************************************
//
// File Name	: 'fat.c'
// Title		: FAT16/32 file system driver
// Author		: Pascal Stang
// Date			: 11/07/2000
// Revised		: 12/12/2000
// Version		: 0.3
// Target MCU	: ATmega103 (should work for Atmel AVR Series)
// Editor Tabs	: 4
//
// NOTE: This code is currently below version 1.0, and therefore is considered
// to be lacking in some functionality or documentation, or may not be fully
// tested.  Nonetheless, you can expect most functions to work.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************


#include <io.h>
#include <string.h>
#include <progmem.h>

#include "./ata.h"
#include "uart.h"
#include "rprintf.h"

#include "./fat.h"

// debug on/off
#define DEBUG_FAT

#define DRIVE0 

// globals
unsigned char *SectorBuffer  =		(unsigned char *) buffer;
unsigned char *fat_cache =	(unsigned char *) fatBuffer;

struct partrecord PartInfo;
unsigned char Fat32Enabled;
unsigned long FirstDataSector;
unsigned int  BytesPerSector;
unsigned int  SectorsPerCluster;
unsigned long FirstFATSector;
unsigned long FirstDirSector;
unsigned long FileSize;
unsigned long FatInCache = 0;

struct fileinfo CurrentFile;

/*************************************************************************/
/*************************************************************************/


unsigned long clust2sect(unsigned long clust)
{
  rprintf("Convert clust: ");  
  rprintfNum(16, 4, FALSE, '0', clust);
  rprintf(" S/C: ");  
  rprintfNum(10, 4, FALSE, ' ', SectorsPerCluster);
  rprintf(" First data: ");  
  rprintfNum(10, 4, FALSE, ' ', FirstDataSector);
   rprintf(" Sector: ");  
  rprintfNum(16, 4, FALSE, '0', (((clust-2) * SectorsPerCluster) + FirstDataSector));
  rprintf("\r\n");  
 
  return ((clust-2) * SectorsPerCluster) + FirstDataSector;
}

unsigned int fatClusterSize(void)
{
  // return the number of sectors in a disk cluster
  return SectorsPerCluster;
}

unsigned char fatInit()
{
  //struct partrecord *pr;
  struct bpb710 *bpb;
  long temp = 0;
  // read partition table
  // TODO.... error checking
  ataReadSector(temp, SectorBuffer);
  // map first partition record	
  // save partition information to global PartInfo
  PartInfo = *((struct partrecord *) ((struct partsector *) SectorBuffer)->psPart);
  //	PartInfo = *pr;
  
  // Read the Partition BootSector
  // **first sector of partition in PartInfo.prStartLBA
  ataReadSector( PartInfo.prStartLBA, SectorBuffer );
  bpb = (struct bpb710 *) ((struct bootsector710 *) SectorBuffer)->bsBPB;
  
  // setup global disk constants
  FirstDataSector	= PartInfo.prStartLBA;
  if(bpb->bpbFATsecs)
    {
      // bpbFATsecs is non-zero and is therefore valid
      // always add 32, since data sector is 32 sectors after root directory sector
      FirstDataSector	+= bpb->bpbResSectors + bpb->bpbFATs * bpb->bpbFATsecs + 32;
    }
  else
    {
      // bpbFATsecs is zero, real value is in bpbBigFATsecs
      FirstDataSector	+= bpb->bpbResSectors + bpb->bpbFATs * bpb->bpbBigFATsecs;
    }
  SectorsPerCluster	= bpb->bpbSecPerClust;
  BytesPerSector		= bpb->bpbBytesPerSec;
  FirstFATSector		= bpb->bpbResSectors + PartInfo.prStartLBA;
  
  switch (PartInfo.prPartType)
    {
    case PART_TYPE_DOSFAT16:
    case PART_TYPE_FAT16:
    case PART_TYPE_FAT16LBA:
      FirstDirSector	= PartInfo.prStartLBA + bpb->bpbResSectors + bpb->bpbFATs * bpb->bpbFATsecs;
      // First data sector is always 32 sectors after root directory
      FirstDataSector = FirstDirSector + 32;
      Fat32Enabled = FALSE;
      break;
    case PART_TYPE_FAT32LBA:
    case PART_TYPE_FAT32:
      // bpbRootClust field exists in FAT32 bpb710, but not in lesser bpb's
      FirstDirSector = bpb->bpbRootClust;
      // push data sector pointer to end of root directory area
      // need this? FirstDataSector += (bpb->bpbRootDirEnts)/DIRENTRIES_PER_SECTOR;
      Fat32Enabled = TRUE;
      break;
    default:
      rprintf("Found: No Partition!\r\n");
      //return 1;
      break;
    }
  
  
#ifdef DEBUG_FAT
  switch (PartInfo.prPartType)
    {
    case PART_TYPE_DOSFAT16:
      rprintf("Found: DOSFAT 16\r\n");
      break;
    case PART_TYPE_FAT16:
      rprintf("Found: FAT16\r\n");
      break;
    case PART_TYPE_FAT16LBA:
      rprintf("Found: FAT16 LBA\r\n");
      break;
    case PART_TYPE_FAT32LBA:
      rprintf("Found: FAT32 LBA\r\n");
      break;
    case PART_TYPE_FAT32:
      rprintf("Found: FAT32\r\n");
      //return 1;	
      break;
    default:
      rprintf("Found: No Partition!\r\n");
      //return 1;
      break;
    }
  
  rprintf("First sector    : ");
  rprintfNum(10, 10, FALSE, ' ', PartInfo.prStartLBA);
  rprintf("\r\n");
  rprintf("Size            : ");
  rprintfNum(10, 10, FALSE, ' ', PartInfo.prSize);
  rprintf("\r\n");
  rprintf("bytes/sector    : ");
  rprintfNum(10, 10, FALSE, ' ', bpb->bpbBytesPerSec);
  rprintf("\r\n");
  rprintf("sectors/cluster : ");
  rprintfNum(10, 10, FALSE, ' ', bpb->bpbSecPerClust);
  rprintf("\r\n");
  rprintf("reserved sectors: ");
  rprintfNum(10, 10, FALSE, ' ', bpb->bpbResSectors);
  rprintf("\r\n");
  rprintf("FatSectors      : ");
  rprintfNum(10, 10, FALSE, ' ', bpb->bpbFATsecs);
  rprintf("\r\n");
  rprintf("BigFatSectors   : ");
  rprintfNum(10, 10, FALSE, ' ', bpb->bpbBigFATsecs);
  rprintf("\r\n");
  rprintf("Number of Fats  : ");
  rprintfNum(10, 10, FALSE, ' ', bpb->bpbFATs);
  rprintf("\r\n");
  rprintf("First Fat Sector: ");
  rprintfNum(10, 10, FALSE, ' ', FirstFATSector);
  rprintf("\r\n");
  rprintf("First Data Sect : ");
  rprintfNum(10, 10, FALSE, ' ', FirstDataSector);
  rprintf("\r\n");
  rprintf("First Dir Clust : ");
  rprintfNum(10, 10, FALSE, ' ', FirstDirSector);
  rprintf("\r\n");
#endif
  
  return 0;	
}

//////////////////////////////////////////////////////////////


unsigned int baseentry = 0;
unsigned int entrycount = 0;


// get the starting cluster of the ith entry.  
// if i == 0, then return the number of files.
unsigned long fatGetDirEntry(int i)
{
  unsigned long sector;
  unsigned long cluster=0;
  struct direntry *de = 0;	// avoid compiler warning by initializing
  //struct winentry *we;
  //unsigned int hasBuffer;
  //unsigned int b;
  //int i;
  int index;
  int numFiles=1;
  //char *p;
  
  // read dir data

  sector = FirstDirSector;
  
  index = 16;	// crank it up
  
  do 
    {
      if (index == 16)	// time for next sector ?
	{
	  
	  ataReadSector( sector++, SectorBuffer);
	  de = (struct direntry *) SectorBuffer;
	  index = 0;
	}	
      
      if (*de->deName != 0xE5)	// if not a deleted entry
	{
	  // ignore long filename entries
	  if (de->deAttributes == ATTR_LONG_FILENAME)
	    {
	    }
	  // ignore volume labels
	  else if (de->deAttributes == ATTR_VOLUME)
	    {
	    }
	  // ignore non-mp3 files
	  else if (strncmp(de->deExtension, "MP3", 3) != 0)
	    {
	    }
	  else // short name entry
	    {
	      printDir(de);
	      if (i == numFiles) { 

		rprintf("getting cluster... ");   
		cluster = (unsigned long) de->deHighClust;
		cluster = cluster << 16;
		cluster = cluster | (unsigned long) de->deStartCluster;
		
		rprintf("Cluster num: ");  
		rprintfNum(10, 10, FALSE, ' ', i);
		rprintf(" H:");  
		rprintfNum(16, 4, FALSE, '0', de->deHighClust);
		rprintf(" L:");  
		rprintfNum(16, 4, FALSE, '0', de->deStartCluster);
		rprintf("\r\n");
		
		rprintf("Cluster num returned: ");  
		rprintfNum(10, 10, FALSE, ' ', cluster);
		rprintf("\r\n");  
		
		// init currentFile
		strncpy(&CurrentFile.name, de->deName, 8);
		CurrentFile.size = de->deFileSize;
		CurrentFile.startCluster = cluster;
		CurrentFile.currentCluster = cluster;

		return cluster;
	      } 
	      numFiles++;
	    }
	}
      index++;
      de++;
    }	while (*de->deName);	// 0 in de->deName[0] if no more entries
  
  return numFiles-1;
  
}


// return the size of the last directory entry
unsigned long fatGetFilesize(void)
{
  return FileSize;
}


// return the long name of the last directory entry
char* fatGetFilename(void)
{	
  return 0;//long_name_buffer;
}


// return the directory of the last directory entry
char* fatGetDirname(void)
{	
  return 0;//dir_name_buffer;
}


// load a clusterfull of data
void fatLoadCluster(unsigned long cluster, unsigned char *buffer)
{
  register unsigned char i;
  // read cluster
  //while ( ataReadSector( DRIVE0, clust2sect(cluster), SectorsPerCluster, buffer) != 0);
  for(i=0; i<SectorsPerCluster; i++)
    {
      //		ataReadSector( DRIVE0, clust2sect(cluster)+i, 1, buffer+(i<<9) );
      // temporary fix for wierd misaligned cluster problem
      // (only when using FAT16?)
      ataReadSector( clust2sect(cluster+8)+i, buffer+(i<<9) );
    }
}


// find next cluster in the FAT chain, return 0 if you're the last sector
unsigned long fatNextCluster(unsigned long cluster)
{
  unsigned long nextCluster;
  unsigned long fatMask;
  unsigned long fatOffset;
  unsigned long sector;
  unsigned int offset;

#ifdef DEBUG_FAT
  //  rprintf("For: ");  
  //rprintfNum(16, 4, FALSE, '0', cluster);
  //rprintf(", ");
#endif
  
  // get fat offset in bytes
  
  // two FAT bytes (16 bits) for every cluster
  fatOffset = cluster << 1;
  // set the FAT bit mask
  fatMask = FAT16_MASK;
  
  // calculate the FAT sector that we're interested in
  sector = FirstFATSector + (fatOffset / BytesPerSector);
  // calculate offset of the our entry within that FAT sector
  offset = fatOffset % BytesPerSector;
  
  // if we don't already have this FAT chunk loaded, go get it
  if (sector != FatInCache)
    {
      // read sector of FAT table
      while (ataReadSector( sector, (unsigned char*)fat_cache) != 0);
      FatInCache = sector;
    }
  
  // read the nextCluster value
  nextCluster = (*((unsigned long*) &((char*)fat_cache)[offset])) & fatMask;
  
  // check to see if we're at the end of the chain
  if (nextCluster == (CLUST_EOFE & fatMask))
    nextCluster = 0;
  
#ifdef DEBUG_FAT
  //  rprintf("next: ");  
  //rprintfNum(16, 4, FALSE, '0', nextCluster);
  //rprintf("\r\n");
#endif
  
  return nextCluster;
}

// dump Nth Sector into memory
void getNthSector(struct fileinfo*, unsigned long N);

// get the TAG of the current file
void getTAG() {
  unsigned long currentCluster =  CurrentFile.currentCluster;
  unsigned long nextCluster = CurrentFile.currentCluster;
  unsigned long lastCluster;
  unsigned long sectorNum;
  u08 sectorOffset;
  unsigned int offset; 
  u08 firstSectorBytes;
  u08 secondSectorBytes;
  struct ID3v1dot1 tag;
  if ((CurrentFile.size % 512) < 128) {
    // TAG is across 2 sectors
    rprintf("Need to read 2 sectors");
    while (nextCluster != 0) {
      lastCluster = currentCluster;
      currentCluster = nextCluster;
      nextCluster = fatNextCluster(currentCluster);
    }
    sectorOffset = (CurrentFile.size % (512*SectorsPerCluster))/512;
    offset = 512 + (CurrentFile.size % 512);
    offset -= 128;
    firstSectorBytes = (512 - offset);
    secondSectorBytes = 128 - (512 - offset);
    if (sectorOffset == 0) {
      // need to go back a cluster
      sectorNum = clust2sect(lastCluster);
      sectorNum += SectorsPerCluster - 1;
    }
    else {
      sectorNum = clust2sect(currentCluster);
      sectorNum += sectorOffset;
    }
    ataReadSector(sectorNum, buffer);
    memorycpy(&tag, &buffer[offset], firstSectorBytes);
    
    if (sectorOffset == 0) {
      // need to go back a cluster
      sectorNum = clust2sect(currentCluster);
    }
    else sectorNum++;
    ataReadSector(sectorNum, buffer);
    memorycpy((&tag+firstSectorBytes), buffer, secondSectorBytes);
  }
  
  else {
    while (nextCluster != 0) {
      currentCluster = nextCluster;
      nextCluster = fatNextCluster(currentCluster);
    }
    sectorNum = clust2sect(currentCluster);
    sectorOffset = (CurrentFile.size % (512*SectorsPerCluster))/512;
    offset = (CurrentFile.size % 512) - 128;
    
    sectorNum += sectorOffset;
    ataReadSector(sectorNum, buffer);
    
    memorycpy(&tag, &buffer[offset], sizeof(tag));
  }

  rprintf("%c", tag.tagID[0]);
  rprintf("%c", tag.tagID[1]);
  rprintf("%c", tag.tagID[2]);

  if (strncmp(tag.tagID, "TAG", 3) == 0) {
    rprintf("TAG FOUND\r\n");
    
    memorycpy(&CurrentFile.title, tag.title, 30);
    memorycpy(&CurrentFile.artist, tag.artist, 30);
    memorycpy(&CurrentFile.comment, tag.comment, 30);
    fixString(&CurrentFile.title, 30);
    fixString(&CurrentFile.artist, 30);
    fixString(&CurrentFile.comment, 30);

    
  }
  else rprintf("NO TAG\r\n");
  
  //ataPrintSector(buffer);
}


// print current file info
void printFileInfo() {
  u08 i;
  rprintf("FILE: ");
  for (i=0; i<8; i++) {
    rprintf("%c", CurrentFile.name[i]);
  }
  rprintf(" SIZE: ");
  rprintfNum(10, 10, FALSE, ' ', CurrentFile.size);
  
  rprintf(" START CLUST: ");
  rprintfNum(16, 8, FALSE, '0', CurrentFile.currentCluster);
  
  rprintf("\r\n");

  rprintf("Title: ");
  printString(&CurrentFile.title);
  rprintf("\r\n");
  rprintf("Artist: ");
  printString(&CurrentFile.artist);
  rprintf("\r\n");
  rprintf("Comments: ");
  printString(&CurrentFile.comment);
  rprintf("\r\n");


}


// print current file info
u08 printFileInfoLCD() {
  u08 size= 2;
  lcdSetAddress(0x40);
  if (stringLen(&CurrentFile.title) > 19) {
    printsubString(&CurrentFile.title, 19);
    size += 19;
  }
  else {
    printString(&CurrentFile.title);
    size += stringLen(&CurrentFile.title);  
  }
  rprintf("-");
  if (stringLen(&CurrentFile.artist) > 19) {
    printsubString(&CurrentFile.artist, 19);
    size += 19;
  }
  else {
    printString(&CurrentFile.artist);
    size += stringLen(&CurrentFile.artist);  
  }
  if (size < 24) return 0;
  while (size < 40) {
    rprintf(" ");
    size ++;
  }
  return 1;
}



void printDir(struct direntry* dir) {
  unsigned long i;
  for (i=0; i<8; i++) {
    rprintf("%c",dir->deName[i]);
  }
  
  rprintf(".");    
  
  for (i=0; i<3; i++) {
    rprintf("%c",dir->deExtension[i]);
  }
  rprintf("\r\n");
  if (dir->deAttributes & ATTR_NORMAL)
    rprintf("NORMAL\r\n");
  if (dir->deAttributes & ATTR_READONLY)
    rprintf("READONLY\r\n");
  if (dir->deAttributes & ATTR_HIDDEN)
    rprintf("HIDDEN\r\n");
  if (dir->deAttributes & ATTR_SYSTEM)
    rprintf("SYSTEM\r\n");
  if (dir->deAttributes & ATTR_VOLUME)
    rprintf("VOLUME\r\n");
  
  rprintf("CREATED: %d/%d/%d ", 
	  ((dir->deCDate[0] >> 5) & 0x07) | ((dir->deCDate[1] & 0x01) << 3),
	  dir->deCDate[0] & 0x1F,
	  (dir->deCDate[1] >> 1) + 1980
	  );
  printTime((dir->deCTime[1] >> 3), 
	    ((dir->deCTime[0] >> 5) & 0x07) | ((dir->deCTime[1] & 0x07) << 3),
	    (dir->deCTime[0] & 0x1F)*2);
  rprintf("\r\n");

  i = dir->deHighClust;
  i = i << 16;
  i = i | dir->deStartCluster;
  rprintf("Cluster num: ");  
  rprintfNum(10, 10, FALSE, ' ', i);
  rprintf(" H:");  
  rprintfNum(16, 4, FALSE, '0', dir->deHighClust);
  rprintf(" L:");  
  rprintfNum(16, 4, FALSE, '0', dir->deStartCluster);
  rprintf("\r\n");

  rprintf("File size: ");
  rprintfNum(10, 10, FALSE, ' ', dir->deFileSize);
  rprintf("\r\n");
  
  rprintf("\r\n");
  return;
}





